@@ -2,20 +2,20 @@
 # Maximum Likelihood Functions for gmnl
 #########################################
 
-## suml function from mlogit (Croissant, 2013)
+## suml function from mlogit (Croissant, 2013) ====
 suml <- function(x){
   n <- length(x)
-  if (!is.null(dim(x[[1]]))){
+  if (!is.null(dim(x[[1]]))) {
     d <- dim(x[[1]])
     s <- matrix(0,d[1], d[2])
-    for (i in 1:n){
+    for (i in 1:n) {
       x[[i]][is.na(x[[i]])] <- 0
       s <- s + x[[i]]
     }
   }
   else{
     s <- rep(0, length(x[[n]]))
-    for (i in 1:n){
+    for (i in 1:n) {
       x[[i]][is.na(x[[i]])] <- 0
       s <- s + x[[i]]
     }
@@ -23,19 +23,19 @@ suml <- function(x){
   s
 }
 
-## suml function for array
+## suml function for array ====
 suml.array <- function(x){
   n <- length(x)
   d <- dim(x[[1]])
   s <- array(0, dim = c(d[1], d[2], d[3]))
-  for (i in 1:n){
+  for (i in 1:n) {
     x[[i]][is.na(x[[i]])] <- 0
     s <- s + x[[i]]
   }
   s
 }
 
-## Multinomial Model based on mlogit (Croissant, 2013)
+## Multinomial Model based on mlogit (Croissant, 2013) ====
 ll.mlogit <- function(theta, y, X, gradient = TRUE, 
                       hessian =  TRUE, weights = NULL, 
                       get.bi = FALSE){
@@ -49,7 +49,7 @@ ll.mlogit <- function(theta, y, X, gradient = TRUE,
   ll <- sum(log(Pn) * weights) 
   
   # Gradient
-  if (gradient | hessian ){
+  if (gradient | hessian ) {
     Px <- suml(mapply("*", X, Pni, SIMPLIFY = FALSE)) 
     yx <- suml(mapply("*", X, y, SIMPLIFY = FALSE))
     gradi <- (yx - Px) * weights ; colnames(gradi) <- names(theta)
@@ -57,15 +57,15 @@ ll.mlogit <- function(theta, y, X, gradient = TRUE,
     attr(ll, "gradient") <- gradi
   }
   # Hessian
-  if (hessian){
+  if (hessian) {
     dxpx <- lapply(X, function(x){
       d <- x - Px ;
       d[is.na(d)] <- 0 ;
       d})
-    hess <- - suml(mapply(function(x, y) crossprod(x * y, y), Pni, dxpx , SIMPLIFY = FALSE))
+    hess <- -suml(mapply(function(x, y) crossprod(x * y, y), Pni, dxpx , SIMPLIFY = FALSE))
     attr(ll, "hessian")
   }
-  if (get.bi){
+  if (get.bi) {
     Pni <- Reduce("cbind", Pni)
     colnames(Pni) <- names(y)
     attr(ll, "prob.alt") <- Pni
@@ -74,7 +74,7 @@ ll.mlogit <- function(theta, y, X, gradient = TRUE,
   ll
 }
 
-## Simulated Maximum Likelihood for SMNL model
+## Simulated Maximum Likelihood for SMNL model ====
 ll.smlogit <- function(theta, y, X, H = NULL, id = NULL, R,
                        weights = NULL, seed = 12345, notscale = NULL,
                        bound.err, gradient = TRUE, typeR = TRUE,
@@ -84,7 +84,7 @@ ll.smlogit <- function(theta, y, X, H = NULL, id = NULL, R,
   J <- length(X)
   N <- nrow(X[[1]])
   panel <- !is.null(id)
-  if (panel){
+  if (panel) {
     n <- length(unique(id))
     if (length(weights) == 1) weights <- rep(weights, N)
   } 
@@ -97,9 +97,9 @@ ll.smlogit <- function(theta, y, X, H = NULL, id = NULL, R,
   
   ## Make random draws
   set.seed(seed)
-  epsilon <- if (typeR) truncnorm::rtruncnorm(R * ifelse(panel, n, N), a = - bound.err, b = bound.err) 
+  epsilon <- if (typeR) truncnorm::rtruncnorm(R * ifelse(panel, n, N), a = -bound.err, b = bound.err) 
              else Make.epsi(R * ifelse(panel, n, N), bound.err)
-  sigmaM  <- exp(tau * matrix(epsilon, ncol = ifelse(panel, n, N))) 
+  sigmaM  <- exp(tau * matrix(epsilon, ncol = ifelse(panel, n, N))) #normalization
   sum.se  <- apply(sigmaM * matrix(epsilon, ncol = ifelse(panel, n, N)), 1, sum) 
   p.se    <- sum.se / apply(sigmaM, 1, sum) 
   sigbar  <- -log(apply(sigmaM , 1, mean))
@@ -111,18 +111,18 @@ ll.smlogit <- function(theta, y, X, H = NULL, id = NULL, R,
   if (panel) theIds <- unique(id)
   if (get.bi) bi <- array(NA, dim = c(nind, R, K), 
                           dimnames = list(NULL, NULL, names(beta))) 
-  for (i in 1:nind){
-    if (panel){
+  for (i in 1:nind) {
+    if (panel) {
       anid <- theIds[i]
       theRows <- which(id == anid)
     }
     else theRows <- i 
     err <- epsilon[((i - 1) * R + 1):(i * R)]
-    if (het){ 
+    if (het) { 
       sigma.nr <- exp(sigbar +  drop(tcrossprod(H[i, , drop = FALSE], t(delta))) + tau * err) 
     } else sigma.nr <- exp(sigbar + tau * err)
     br <- matrix(NA, K, R)
-    for(k in 1:K){
+    for (k in 1:K) {
       ns <- notscale[k]
       if (ns == 0) br[k, ] <- beta[k] * sigma.nr else br[k, ] <- beta[k]
     } 
@@ -140,7 +140,7 @@ ll.smlogit <- function(theta, y, X, H = NULL, id = NULL, R,
   lnL <- if (panel) sum(log(Pn) * weights[!duplicated(id)]) else sum(log(Pn) * weights)
   
   ## Make Gradient
-  if (gradient){
+  if (gradient) {
     lambda <- mapply(function(y, p) y - p, y, Pntir, SIMPLIFY =  FALSE) # J-list NT * R
     Qnr    <-  Pnr / (Pn * R) # N * R
     if (panel) Qnr <-  Qnr[as.character(id), ] # NT * R
@@ -150,25 +150,25 @@ ll.smlogit <- function(theta, y, X, H = NULL, id = NULL, R,
     if (het) dUdd <- vector(mode = 'list', length = J)
     d.mu <- d.tau <- matrix(NA, K, R)
     if (het) d.delta <- matrix(NA, K, R)
-    for (j in 1:J){
+    for (j in 1:J) {
       dUdt[[j]] <- dUdb[[j]] <- matrix(NA, N, K)
       if (het) dUdd[[j]] <- matrix(NA, N, K)
     }
-    for (i in 1:nind){
-      if (panel){
+    for (i in 1:nind) {
+      if (panel) {
         anid <- theIds[i]
         theRows <- which(id == anid)
       }
       else theRows <- i
       err <- epsilon[((i - 1) * R + 1):(i * R)]
-      if (het){ 
+      if (het) { 
         sigma.nr <- exp(sigbar +  drop(tcrossprod(H[i, , drop = FALSE], t(delta))) + tau * err) 
       } else sigma.nr <- exp(sigbar + tau * err)
-      for(k in 1:K){
+      for (k in 1:K) {
         ns <- notscale[k]
-        if(ns == 0){
+        if (ns == 0) {
           d.mu[k, ]  <- sigma.nr
-          d.tau[k, ] <- sigma.nr * (- p.se + err) * beta[k]
+          d.tau[k, ] <- sigma.nr * (-p.se + err) * beta[k]
           if (het) d.delta[k, ] <- sigma.nr * beta[k]
         } else {
           d.mu[k, ] <- 1 
@@ -185,9 +185,9 @@ ll.smlogit <- function(theta, y, X, H = NULL, id = NULL, R,
     tempt  <- mapply("*", dUdt, X, SIMPLIFY =  FALSE)
     gbart  <- suml(lapply(tempt, function(x) rowSums(x)))
     gbarmi <- suml(mapply("*", X, dUdb, SIMPLIFY =  FALSE))
-    if (het){
+    if (het) {
       tempd  <- mapply("*", dUdd, X, SIMPLIFY =  FALSE)
-      if(panel) H <- H[id, ]
+      if (panel) H <- H[id, ]
       gbarde <- suml(lapply(tempd, function(x) rowSums(x))) * H
       gbari  <- cbind(gbarmi, gbart, gbarde)
     } else {
@@ -215,7 +215,7 @@ ll.mixlog <- function(theta, y, X, Z = NULL, id = NULL, ranp, R,
   J <- length(X)
   N <- nrow(X[[1]])
   panel <- !is.null(id)
-  if (panel){
+  if (panel) {
     n <- length(unique(id))
     if (length(weights) == 1) weights <- rep(weights, N)
   } 
@@ -223,7 +223,7 @@ ll.mixlog <- function(theta, y, X, Z = NULL, id = NULL, ranp, R,
   
   ## Get variables
   Vara <- sort(match(names(ranp), colnames(X[[1]])))
-  Varc <- (1:K)[- Vara]
+  Varc <- (1:K)[-Vara]
   Ka   <- length(Vara)
   Kc   <- length(Varc)
   fixed <- !(Kc == 0)
@@ -234,14 +234,14 @@ ll.mixlog <- function(theta, y, X, Z = NULL, id = NULL, ranp, R,
   ## Get Parameters
   if (fixed) gamma <- theta[colnames(Xc[[1]])]
   beta.bar <- theta[colnames(Xa[[1]])]
-  if (hier){
+  if (hier) {
     # Parameters
     hname  <- unlist(lapply(names(mvar), function(x) outer(x, mvar[[x]], FUN = paste, sep = ".")))
     phi <- theta[hname]
     sigma <- theta[-c(1:(K + length(hname)))] 
     
     # Variables
-    thecolumns <- lapply(mvar, function (x) {sort(match(x, colnames(Z)))}) 
+    thecolumns <- lapply(mvar, function(x) {sort(match(x, colnames(Z)))}) 
     Zlist  <- lapply(thecolumns, function(x) Z[, x, drop = FALSE]) # Z variables in a list
   } else {
     sigma <- theta[-c(1:K)]
@@ -265,8 +265,8 @@ ll.mixlog <- function(theta, y, X, Z = NULL, id = NULL, ranp, R,
   if (panel) theIds <- unique(id)
   if (get.bi) bi <- array(NA, dim = c(nind, R, Ka), 
                           dimnames = list(NULL, NULL, names(beta.bar))) 
-  for (i in 1:nind){
-    if (panel){
+  for (i in 1:nind) {
+    if (panel) {
       anid <- theIds[i]
       theRows <- which(id == anid)
     }
@@ -281,7 +281,7 @@ ll.mixlog <- function(theta, y, X, Z = NULL, id = NULL, ranp, R,
     if (get.bi) bi[i, , ] <- t(b$br)
   }
   ## Make log L
-  if (fixed){
+  if (fixed) {
     EXB <- mapply(function(x, y) exp(x + y), XBf, XBr, SIMPLIFY = FALSE)
   } else EXB <- lapply(XBr, function(x) exp(x))
   
@@ -294,21 +294,21 @@ ll.mixlog <- function(theta, y, X, Z = NULL, id = NULL, ranp, R,
   lnL <- if (panel) sum(log(Pn) * weights[!duplicated(id)]) else sum(log(Pn) * weights)
   
   ## Make Gradient
-  if (gradient){
+  if (gradient) {
     lambda <- mapply(function(y, p) y - p, y, Pntir, SIMPLIFY =  FALSE) 
     Qnr  <-  Pnr / (Pn * R) 
-    if(panel) Qnr <- Qnr[as.character(id), ] 
+    if (panel) Qnr <- Qnr[as.character(id), ] 
     eta  <- lapply(lambda, function(x) x * Qnr)
     
     dUdb <- dUds <- vector(mode = 'list', length = J)
     if (hier) dUdphi <- vector(mode = 'list', length = J)
-    for (j in 1:J){
+    for (j in 1:J) {
       dUdb[[j]] <- matrix(NA, N, Ka)
       dUds[[j]] <- matrix(NA, N, length(sigma))
       if (hier) dUdphi[[j]] <- matrix(NA, N, length(phi))
     }
-    for (i in 1:nind){
-      if (panel){
+    for (i in 1:nind) {
+      if (panel) {
         anid <- theIds[i]
         theRows <- which(id == anid)
       }
@@ -322,9 +322,9 @@ ll.mixlog <- function(theta, y, X, Z = NULL, id = NULL, ranp, R,
         if (hier) dUdphi[[j]][theRows, ] <- tcrossprod(eta[[j]][theRows, , drop = FALSE], b$d.pis)
       }   
     }
-    if (correlation){
+    if (correlation) {
       vecX <- c()
-      for (i in 1:Ka){
+      for (i in 1:Ka) {
         vecX <- c(vecX, i:Ka)
       }
       Xac <- lapply(Xa,  function(x) x[, vecX])
@@ -332,7 +332,7 @@ ll.mixlog <- function(theta, y, X, Z = NULL, id = NULL, ranp, R,
       Xac <- Xa  
     }
     
-    if (fixed){
+    if (fixed) {
       seta <- lapply(eta, function(x) rowSums(x))
       gbarfi <- suml(mapply("*", Xc, seta, SIMPLIFY =  FALSE))
     } else gbarfi <- numeric()
@@ -377,7 +377,7 @@ ll.gmlogit <- function(theta, y, X, H = NULL, id = NULL, ranp, R,
   
   ## Get variables
   Vara <- sort(match(names(ranp), colnames(X[[1]])))
-  Varc <- (1:K)[- Vara]
+  Varc <- (1:K)[-Vara]
   Ka   <- length(Vara)
   Kc   <- length(Varc)
   fixed <- !(Kc == 0)
@@ -388,7 +388,7 @@ ll.gmlogit <- function(theta, y, X, H = NULL, id = NULL, ranp, R,
   ## Get parameters
   if (fixed) delta <- theta[colnames(Xc[[1]])]
   beta.bar <- theta[colnames(Xa[[1]])]
-  if (het){
+  if (het) {
     names.het <- paste("het", colnames(H), sep = ".")
     het.par <- theta[names.het]
     stds <- head(theta[-c(1:(K + length(names.het)))], -2L) 
@@ -399,11 +399,11 @@ ll.gmlogit <- function(theta, y, X, H = NULL, id = NULL, ranp, R,
   ## Make random draws
   set.seed(seed)
   Omega <- make.draws(R * ifelse(panel, n, N), Ka, haltons)
-  epsilon <- if (typeR) truncnorm::rtruncnorm(R * ifelse(panel, n, N), a = - bound.err, b = bound.err) else Make.epsi(R * ifelse(panel, n, N), bound.err)
+  epsilon <- if (typeR) truncnorm::rtruncnorm(R * ifelse(panel, n, N), a = -bound.err, b = bound.err) else Make.epsi(R * ifelse(panel, n, N), bound.err)
   sigmaM  <- exp(tau * matrix(epsilon, ncol = ifelse(panel, n, N)))
   sum.se  <- apply(sigmaM * matrix(epsilon, ncol = ifelse(panel, n, N)), 1, sum) 
   p.se    <- sum.se / apply(sigmaM, 1, sum) 
-  sigbar  <- - log(apply(sigmaM , 1, mean))
+  sigbar  <- -log(apply(sigmaM , 1, mean))
   
   ## Make utility for each individual
   XBr <- XBf <- vector(mode = 'list', length = J)
@@ -412,24 +412,24 @@ ll.gmlogit <- function(theta, y, X, H = NULL, id = NULL, ranp, R,
   if (panel) theIds <- unique(id)
   if (get.bi) bi <- array(NA, dim = c(nind, R, Ka + Kc), 
                           dimnames = list(NULL, NULL, colnames(X[[1]]))) 
-  for (i in 1:nind){
-    if (panel){
+  for (i in 1:nind) {
+    if (panel) {
       anid <- theIds[i]
       theRows <- which(id == anid)
     }
     else theRows <- i
     err <- epsilon[((i - 1) * R + 1):(i * R)]
-    if (het){
+    if (het) {
       Hb <- drop(tcrossprod(H[i, , drop = FALSE], t(het.par)))
       sigma.nr <- exp(sigbar + Hb + tau * err) 
     } else {
       sigma.nr <- exp(sigbar + tau * err)
       Hb <- NULL
     }    
-    if (fixed){
+    if (fixed) {
       br <- matrix(NA, Kc, R)
       rownames(br) <- names(delta)
-      for(k in 1:Kc){
+      for (k in 1:Kc) {
         ns <- notscale[names(delta[k])]
         if (ns == 0) br[k, ] <- delta[k] * sigma.nr else br[k, ] <- delta[k]
       } 
@@ -443,10 +443,10 @@ ll.gmlogit <- function(theta, y, X, H = NULL, id = NULL, ranp, R,
       XBr[[j]][theRows, ] <- crossprod(t(Xa[[j]][theRows, , drop = FALSE]), b$br) 
       if (fixed) XBf[[j]][theRows, ] <- crossprod(t(Xc[[j]][theRows, , drop = FALSE]), br) 
     }
-    if (get.bi) bi[i, , ] <- if(fixed) t(rbind(br, b$br)) else t(b$br)
+    if (get.bi) bi[i, , ] <- if (fixed) t(rbind(br, b$br)) else t(b$br)
   }
   ## Make log L
-  if (fixed){
+  if (fixed) {
     EXB <- mapply(function(x, y) exp(x + y), XBf, XBr, SIMPLIFY = FALSE)
   } else {
     EXB <- lapply(XBr, function(x) exp(x))
@@ -460,14 +460,14 @@ ll.gmlogit <- function(theta, y, X, H = NULL, id = NULL, ranp, R,
   lnL <- if (panel) sum(log(Pn) * weights[!duplicated(id)]) else sum(log(Pn) * weights)
   
   ## Make Gradient
-  if (gradient){
+  if (gradient) {
     lambda <- mapply(function(y, p) y - p, y, Pntir, SIMPLIFY =  FALSE) 
     Qnr  <-  Pnr / (Pn * R) 
-    if(panel) Qnr <-  Qnr[as.character(id), ] 
+    if (panel) Qnr <-  Qnr[as.character(id), ] 
     eta  <- lapply(lambda, function(x) x * Qnr)
     
     dUdf <- dUdb <- dUds <- dUdt <- dUdg <- dUdh <- vector(mode = 'list', length = J)
-    for (j in 1:J){
+    for (j in 1:J) {
       dUdt[[j]] <- matrix(NA, N, K) 
       dUdg[[j]] <- matrix(NA, N, Ka)  
       dUdb[[j]] <- matrix(NA, N, Ka)
@@ -475,27 +475,27 @@ ll.gmlogit <- function(theta, y, X, H = NULL, id = NULL, ranp, R,
       if (het)   dUdh[[j]] <- matrix(NA, N, K)
       dUds[[j]] <- matrix(NA, N, length(stds))
     }
-    for (i in 1:nind){
-      if (panel){
+    for (i in 1:nind) {
+      if (panel) {
         anid <- theIds[i]
         theRows <- which(id == anid)
       }
       else theRows <- i
       err <- epsilon[((i - 1) * R + 1):(i * R)]
-      if (het){
+      if (het) {
         Hb <- drop(tcrossprod(H[i, , drop = FALSE], t(het.par)))
         sigma.nr <- exp(sigbar + Hb + tau * err) 
       } else {
         sigma.nr <- exp(sigbar + tau * err)
         Hb <- NULL
       }  
-      if (fixed){
+      if (fixed) {
         d.muf <- d.tauf <- d.hetf <- matrix(NA, Kc, R)
-        for(k in 1:Kc){
+        for (k in 1:Kc) {
           ns <- notscale[names(delta[k])] # Look for name
           if (ns == 0) {
             d.muf[k, ]  <- sigma.nr
-            d.tauf[k, ] <- sigma.nr * (- p.se + err) * delta[k]
+            d.tauf[k, ] <- sigma.nr * (-p.se + err) * delta[k]
             if (het) d.hetf[k, ] <- sigma.nr * delta[k]
           } else {
             d.muf[k, ]  <- 1
@@ -510,7 +510,7 @@ ll.gmlogit <- function(theta, y, X, H = NULL, id = NULL, ranp, R,
                      epsilon = err, sigbar = sigbar, p.se = p.se , 
                      hgamma = hgamma, notscale =  notscale, Hb = Hb)
       for (j in 1:J) {
-        if (fixed){
+        if (fixed) {
           dUdf[[j]][theRows, ] <- tcrossprod(eta[[j]][theRows, , drop = FALSE], d.muf)
           dUdt[[j]][theRows, ] <- cbind(tcrossprod(eta[[j]][theRows, , drop = FALSE], d.tauf), 
                                         tcrossprod(eta[[j]][theRows, , drop = FALSE], b$d.tau))
@@ -525,9 +525,9 @@ ll.gmlogit <- function(theta, y, X, H = NULL, id = NULL, ranp, R,
         dUds[[j]][theRows, ] <- tcrossprod(eta[[j]][theRows, , drop = FALSE], b$d.stds)
       }   
     }
-    if (correlation){
+    if (correlation) {
       vecX <- c()
-      for (i in 1:Ka){
+      for (i in 1:Ka) {
         vecX <- c(vecX, i:Ka)
       }
       Xac <- lapply(Xa,  function(x) x[, vecX])
@@ -535,19 +535,19 @@ ll.gmlogit <- function(theta, y, X, H = NULL, id = NULL, ranp, R,
       Xac <- Xa  
     }
     
-    if (fixed){
+    if (fixed) {
       tempt <- mapply(function(x, y, z) x * cbind(y, z), dUdt, Xc, Xa, SIMPLIFY =  FALSE)
     } else {
       tempt <- mapply(function(x, y) x * y, dUdt, Xa, SIMPLIFY =  FALSE)
     }
     gbart <-  suml(lapply(tempt, function(x) rowSums(x))) 
-    if(het){ 
-      if (fixed){
+    if (het) { 
+      if (fixed) {
         temph <- mapply(function(x, y, z) x * cbind(y, z), dUdh, Xc, Xa, SIMPLIFY =  FALSE)
       } else {
         temph <- mapply(function(x, y) x * y, dUdh, Xa, SIMPLIFY =  FALSE)
       }
-      if(panel) H <- H[id, ]
+      if (panel) H <- H[id, ]
       ghet <- suml(lapply(temph, function(x) rowSums(x))) * H
     } else ghet <- numeric()
     tempg <- mapply("*", dUdg, Xa, SIMPLIFY =  FALSE)
@@ -568,7 +568,7 @@ ll.gmlogit <- function(theta, y, X, H = NULL, id = NULL, ranp, R,
   lnL 
 }
 
-## Likelihood function for Latent class model
+## Likelihood function for Latent class model ====
 ll.mlogitlc <- function(theta, y, X, H, Q, id = NULL, weights = NULL,
                         gradient = TRUE, get.bi =  FALSE){
   # Parameters and globals
@@ -608,7 +608,7 @@ ll.mlogitlc <- function(theta, y, X, H, Q, id = NULL, weights = NULL,
   
   
   ## Gradient
-  if (gradient){
+  if (gradient) {
     lambda <- mapply(function(y, p) y - p, y, Pnjq, SIMPLIFY =  FALSE) # J list of N * Q
     Qnr  <-  WPnq / Ln # N * Q
     if (panel) Qnr <- Qnr[id, ] 
@@ -624,13 +624,14 @@ ll.mlogitlc <- function(theta, y, X, H, Q, id = NULL, weights = NULL,
     Wg <- vector(mode = "list", length = Q)
     IQ <- diag(Q)
     #for(q in 1:Q) Wg[[q]] <- matrix(NA, N, 1)
-    for(q in 1:Q) Wg[[q]]<-  rowSums(Qnr * (repRows(IQ[q, ], N) - repCols(Wnq[, q], Q)))
+    for (q in 1:Q) Wg[[q]] <- rowSums(Qnr * (repRows(IQ[q, ], N) - repCols(Wnq[, q], Q)))
     grad.gamma <- suml(mapply("*", H, Wg, SIMPLIFY = FALSE)) 
     gari <- cbind(grad.beta, grad.gamma)
     colnames(gari) <- names(theta)
     attr(lnL, "gradient") <- gari * weights
   }
   if (get.bi) {
+    attr(lnL, 'Wnq') <- Wnq
     if (panel) Wnq <- Wnq[id, ]
     Pw <- lapply(Pnjq, function(x) x * Wnq)
     attr(lnL, "prob.alt") <- sapply(Pw, function(x) apply(x, 1, sum))
@@ -651,7 +652,7 @@ ll.mnlogit <- function(theta, y, X, H, Q,
   J <- length(X)
   N <- nrow(X[[1]])
   panel <- !is.null(id)
-  if (panel){
+  if (panel) {
     n <- length(unique(id))
     if (length(weights) == 1) weights <- rep(weights, N)
   }  
@@ -682,13 +683,13 @@ ll.mnlogit <- function(theta, y, X, H, Q,
   if (panel) theIds <- unique(id)
   if (get.bi) bi <- array(NA, dim = c(nind, R, Q, K), 
                           dimnames = list(NULL, NULL, NULL, colnames(X[[1]]))) 
-  for (i in 1:nind){
-    if (panel){
+  for (i in 1:nind) {
+    if (panel) {
       anid <- theIds[i]
       theRows <- which(id == anid)
     }
     else theRows <- i
-    for (q in 1:Q){
+    for (q in 1:Q) {
       bq <- Makeh.rcoef(beta[, q], stds[, q], ranp, Omega[ , ((i - 1) * R + 1):(i * R), drop = FALSE], 
                        correlation, Pi = NULL, Slist = NULL, mvar = NULL)
       for (j in 1:J) {
@@ -710,7 +711,7 @@ ll.mnlogit <- function(theta, y, X, H, Q,
   lnL <- if (panel) sum(log(Ln) * weights[!duplicated(id)]) else sum(log(Ln) * weights)
   
   ## Gradient
-  if (gradient){
+  if (gradient) {
     lambda <- mapply(function(y, p) y - p, y, Pntirq, SIMPLIFY =  FALSE) # J list of N * R * Q
     Wnq.mod  <- aperm(repmat(Wnq / Ln, dimen = c(1, 1, R)), c(1, 3, 2)) # n * R * Q
     Qnq.mod  <-  Wnq.mod * Pnrq # n * R * Q
@@ -718,18 +719,18 @@ ll.mnlogit <- function(theta, y, X, H, Q,
     eta  <- lapply(lambda, function(x) x * Qnq.mod) # J list of N * R * Q
     
     dUdb <- dUds <- vector(mode = 'list', length = J)
-    for (j in 1:J){
+    for (j in 1:J) {
       dUdb[[j]] <- array(NA, dim = c(N, K, Q))
       dUds[[j]] <- array(NA, dim = c(N, nrow(stds), Q))
     }
     
-    for (i in 1:nind){
-      if (panel){
+    for (i in 1:nind) {
+      if (panel) {
         anid <- theIds[i]
         theRows <- which(id == anid)
       }
       else theRows <- i
-      for (q in 1:Q){
+      for (q in 1:Q) {
         bq <- Makeh.rcoef(beta[, q], stds[, q], ranp, Omega[ , ((i - 1) * R + 1):(i * R), drop = FALSE], 
                           correlation, Pi = NULL, Slist = NULL, mvar = NULL)
         for (j in 1:J) {
@@ -739,9 +740,9 @@ ll.mnlogit <- function(theta, y, X, H, Q,
       } 
     }
     
-    if (correlation){
+    if (correlation) {
       vecX <- c()
-      for (i in 1:K){
+      for (i in 1:K) {
         vecX <- c(vecX, i:K)
       }
       Xac <- lapply(X,  function(x) x[, vecX])
@@ -765,7 +766,7 @@ ll.mnlogit <- function(theta, y, X, H, Q,
     Wg <- vector(mode = "list", length = Q)
     IQ <- diag(Q)
     #for(q in 1:Q) Wg[[q]] <- matrix(NA, N, 1)
-    for(q in 1:Q) Wg[[q]]<-  rowSums(Qnq * (repRows(IQ[q, ], N) - repCols(Wnq[, q], Q)))
+    for (q in 1:Q) Wg[[q]] <- rowSums(Qnq * (repRows(IQ[q, ], N) - repCols(Wnq[, q], Q)))
     grad.gamma <- suml(mapply("*", H, Wg, SIMPLIFY = FALSE)) 
     
     gari <- cbind(grad.beta, grad.stds, grad.gamma)
